#!/usr/bin/env python

import sys
import os
import argparse
#import inspect
#from pprint import pprint

from pcbnew import *

class KicadUp:
    Name = "KicadUp"
    
    def loadSettings(self):
	self.modelPath = "C:/models3d/"
	self.convertPath = "C:/Program Files/ImageMagick-6.7.8-Q8/convert.exe"
	self.compositePath = "C:/Program Files/ImageMagick-6.7.8-Q8/composite.exe"
	self.gerbvPath = "gerbv"
	self.deleteCommand = "del"
	
	return
    
    def loadDefaults(self):
	self.boardThickness = 1.6
	
	self.boardColor = "0x266844"
	self.tracesColor = "0x208549"
	self.platingColor = "silver"
	self.silkColor = "ivory"
	
	self.imageSuffix = "png"
	self.imageDPI = "600"
	self.imageResultExtension = "jpg"

	return
	
    def __init__(self, projectPath, projectName):
	self.ToUnits = ToMM 
	self.FromUnits = FromMM

	self.loadSettings()
	self.loadDefaults()

	self.projectPath = projectPath
	self.projectName = projectName

	self.imageDirectory = "%s/%s"%(self.projectPath, self.projectName)
	
	pcbFilename = "%s/%s.brd"%(self.projectPath, self.projectName)

	self.pcb = LoadBoard(pcbFilename)
	self.boundingBox = self.pcb.ComputeBoundingBox(True)

	if not os.path.exists("%s/kicadup"%(self.projectPath)):
	    os.mkdir("%s/kicadup"%(self.projectPath))
	    
	self.outputFile = open("%s/kicadup/%s.eup"%(self.projectPath, self.projectName), "w")
	
	return
	
    def __del__(self):
	self.outputFile.close()
	return

    def printHeader(self):
	self.outputFile.write("version;4;4;http://eagleup.wordpress.com/;\n")
	self.outputFile.write("settings;%s;%s;%s;%s;\n"%(self.modelPath, self.convertPath, self.compositePath, self.deleteCommand))
	self.outputFile.write("color;%s;%s;%s;%s;\n"%(self.boardColor, self.tracesColor, self.platingColor, self.silkColor))
	self.outputFile.write("images;%s;%s;%s;%s;%s;\n"%(self.imageDirectory, self.projectName, self.imageSuffix, self.imageDPI, self.imageResultExtension))
	
	return

    def printOutline(self):
	#xp, yp = boundingBox.GetPosition()

	#print "outline;%.6f;%.6f;%.6f;%.6f;%.6f;" % (ToUnits(xp), ToUnits(yp), ToUnits(xp + boundingBox.GetWidth()), ToUnits(yp + boundingBox.GetHeight()), boardThickness)
	width = self.ToUnits(self.boundingBox.GetWidth())
	height = self.ToUnits(self.boundingBox.GetHeight())
	
	self.outputFile.write("outline;%.6f;%.6f;%.6f;%.6f;%.6f;\n" % (0.0, 0.0, width, height, self.boardThickness))
	self.outputFile.write("outline-complete;%.2f;\n"%(self.boardThickness))
	
	return

    def isSMDPad(self, pad):
	sizeX, sizeY = self.ToUnits(pad.GetDrillSize())
	
	if sizeX == 0 and sizeY == 0:
	    return True
	    
	return False

    def printHole(self, pad):
	sizeX, sizeY = self.ToUnits(pad.GetDrillSize())
	
	positionX, positionY = self.ToUnits(pad.GetPosition() - self.boundingBox.GetPosition())
	positionY = self.ToUnits(self.boundingBox.GetHeight()) - positionY

	self.outputFile.write("hole;%.3f;%.3f;%.3f;%.3f;\n"%(positionX, positionY, sizeX / 2.0, self.boardThickness / 2.0))
	
	return
	
    def printVia(self, via):
	positionX, positionY = self.ToUnits(via.GetPosition() - self.boundingBox.GetPosition())
	positionY = self.ToUnits(self.boundingBox.GetHeight()) - positionY

	drillSize = self.ToUnits(via.GetDrillValue())
	#width = item.GetWidth()
	self.outputFile.write("hole;%.3f;%.3f;%.3f;%.3f;\n"%(positionX, positionY, drillSize / 2.0, self.boardThickness / 2.0))

	return
	
    def printHoles(self, skipVias):
	holeCount = 0
	
	for module in self.pcb.GetModules():
	    for pad in module.GetPads():
		#ignore SMT pads which don't have holes
		if self.isSMDPad(pad):
		    continue

		self.printHole(pad)
		
		holeCount += 1

	if skipVias == False:
	    for item in self.pcb.GetTracks():
		if type(item) is not SEGVIA:
		    continue
		
		self.printVia(item)
		
		holeCount += 1

	self.outputFile.write("hole-complete;%s;%d;%s;\n"%("element", holeCount, self.platingColor))

	return
	
    def getModel(self, module):
	
	if module.GetLibRef() == "SM0603":
	    if module.GetReference().startswith("C"):
		return "C0603"
	    else:
		return "R0603"
	
	if module.GetLibRef() == "SM0805":
	    if module.GetReference().startswith("C"):
		return "C0805"
	    else:
		return "R0805"

	if module.GetLibRef() == "SM1206":
	    if module.GetReference().startswith("C"):
		return "C1206"
	    else:
		return "R1206"

	if module.GetLibRef() == "SOIC-18":
	    return "SO18W"
	    
	return module.GetLibRef()
	
    def printModule(self, module):
	positionX, positionY = self.ToUnits(module.GetPosition() - self.boundingBox.GetPosition())
	positionY = self.ToUnits(self.boundingBox.GetHeight()) - positionY

	angle = module.GetOrientation() / 10.0;

	#	if angle + 180.0 >= 360.0:
	#	    angle = angle - 180.0
	#	else:
	#	    angle = angle + 180.0
	    
	#check if the module is on top or on bottom layer (flipped)
	if module.IsOnLayer(15):
	    z = self.boardThickness / 2.0;
	else:
	    z = self.boardThickness / -2.0;

	self.outputFile.write("package;%s;%.3f;%.3f;%.0f;%s;%s;%.3f;\n"%(module.GetReference(), positionX, positionY, angle, module.GetValue(), self.getModel(module), z))
	
	return
	
    def printModules(self):
	#printing the position of the modules
	for module in self.pcb.GetModules():
	    self.printModule(module)
	    
	return

    #generates images from the gerber files
    def generateImages(self):
	src = "%s/%s"%(self.projectPath, self.projectName)
	dst = "%s/kicadup/%s"%(self.projectPath, self.projectName)
	
	#all layers need to contain the edges so that gerbv generates images of the same size for each layer
	os.system("gerbv --dpi %s --border 0 --background \"#FFFFFF\" --foreground \"#000000FF\" --export png --output \"%s_bottom.png\" \"%s-Back.gbl\""%(self.imageDPI, dst, src))
	os.system("gerbv --dpi %s --border 0 --background \"#FFFFFF\" --foreground \"#000000FF\" --export png --output \"%s_bottom_mask.png\" \"%s-Mask_Back.gbs\""%(self.imageDPI, dst, src))
	os.system("gerbv --dpi %s --border 0 --background \"#FFFFFF\" --foreground \"#000000FF\" --export png --output \"%s_bottom_silk.png\" \"%s-SilkS_Back.gbo\""%(self.imageDPI, dst, src))

	os.system("gerbv --dpi %s --border 0 --background \"#FFFFFF\" --foreground \"#000000Ff\" --export png --output \"%s_top.png\" \"%s-Front.gtl\""%(self.imageDPI, dst, src))
	os.system("gerbv --dpi %s --border 0 --background \"#FFFFFF\" --foreground \"#000000FF\" --export png --output \"%s_top_mask.png\" \"%s-Mask_Front.gts\""%(self.imageDPI, dst, src))
	os.system("gerbv --dpi %s --border 0 --background \"#FFFFFF\" --foreground \"#000000FF\" --export png --output \"%s_top_silk.png\" \"%s-SilkS_Front.gto\""%(self.imageDPI, dst, src))

	os.system("gerbv --dpi %s --border 0 --background \"#FFFFFF\" --foreground \"#000000FF\" --export png --output \"%s_outline.png\" \"%s-PCB_Edges.gbr\""%(self.imageDPI, dst, src))
	os.system("gerbv --dpi %s --border 0 --background \"#000000\" --foreground \"#000000FF\" --export png --output \"%s_imagesize.png\" \"%s-PCB_Edges.gbr\""%(self.imageDPI, dst, src))
    
	#removing edges from the other layers
	os.system("composite -transparent-color \"white\" -compose minus \"%s_outline.png\" \"%s_bottom.png\" \"%s_bottom.png\""%(dst, dst, dst))
	os.system("convert -negate \"%s_bottom.png\" \"%s_bottom.png\""%(dst, dst))

	os.system("composite -transparent-color \"white\" -compose minus \"%s_outline.png\" \"%s_bottom_mask.png\" \"%s_bottom_mask.png\""%(dst, dst, dst))
	os.system("convert -negate \"%s_bottom_mask.png\" \"%s_bottom_mask.png\""%(dst, dst))

	os.system("composite -transparent-color \"white\" -compose minus \"%s_outline.png\" \"%s_bottom_silk.png\" \"%s_bottom_silk.png\""%(dst, dst, dst))
	os.system("convert -negate \"%s_bottom_silk.png\" \"%s_bottom_silk.png\""%(dst, dst))

	os.system("composite -transparent-color \"white\" -compose minus \"%s_outline.png\" \"%s_top.png\" \"%s_top.png\""%(dst, dst, dst))
	os.system("convert -negate \"%s_top.png\" \"%s_top.png\""%(dst, dst))

	os.system("composite -transparent-color \"white\" -compose minus \"%s_outline.png\" \"%s_top_mask.png\" \"%s_top_mask.png\""%(dst, dst, dst))
	os.system("convert -negate \"%s_top_mask.png\" \"%s_top_mask.png\""%(dst, dst))

	os.system("composite -transparent-color \"white\" -compose minus \"%s_outline.png\" \"%s_top_silk.png\" \"%s_top_silk.png\""%(dst, dst, dst))
	os.system("convert -negate \"%s_top_silk.png\" \"%s_top_silk.png\""%(dst, dst))

	#add a border to the image for correct size detection by the existing SketchUp import plugin
	os.system("convert -border 1 -bordercolor \"white\" \"%s_imagesize.png\" \"%s_imagesize.png\""%(dst, dst))
    
	return
    
print "KicadUp version 0.1"

argParser = argparse.ArgumentParser()

argParser.add_argument("project_name", help="The name of the Kicad project")
argParser.add_argument("project_path", help="The directory where the project is located")
argParser.add_argument("-b", "--board-thickness", type=float, help="The thickness of the board (in mm) - default 1.6 mm", default=1.6)
argParser.add_argument("-r", "--image-dpi", type=str, choices=["300", "600", "1200"], help="The resolution of the generated images - default 600 dpi", default="600")
argParser.add_argument("-m", "--soldermask-color", type=str, choices=["green", "red", "blue", "black", "white", "yellow"], help="The plating color", default="green")
argParser.add_argument("-p", "--plating-color", type=str, choices=["silver", "gold"], help="The plating color", default="silver")
argParser.add_argument("-s", "--silk-color", type=str, choices=["ivory", "white", "black"], help="The plating color", default="ivory")
argParser.add_argument("-i", "--image-directory", type=str, help="Override the image directory in the output file")
argParser.add_argument("-mp", "--model-path", type=str, help="Override the model path in the output file")
argParser.add_argument("-im", "--imagemagick-path", type=str, help="Override the ImageMagick path in the output file")


args = argParser.parse_args()

boardColors = {"green":"0x266844", "red":"0x98041C", "blue":"0x092356", "black":"0x141414", "white":"0xEEEEEE", "yellow":"0xE6FA05", "custom":"0xFF6600"};
tracesColors = {"green":"0x208549", "red":"0xE2012D", "blue":"0x133775", "black":"0x202020", "white":"0xDDDDDD", "yellow":"0xFFC205", "custom":"0xFF0033"};
platingColors = {"silver":"silver", "gold":"gold"}
silkColors = {"ivory":"ivory", "white":"white", "black":"black"}

ku = KicadUp(args.project_path, args.project_name)

ku.boardThickness = args.board_thickness
ku.imageDPI = args.image_dpi
ku.boardColor = boardColors[args.soldermask_color]
ku.tracesColor = tracesColors[args.soldermask_color]
ku.platingColor = platingColors[args.plating_color]
ku.silkColors = silkColors[args.silk_color]

if args.image_directory is not None:
    ku.imageDirectory = args.image_directory

if args.model_path is not None:
    ku.modelPath = args.model_path

if args.imagemagick_path is not None:
    ku.convertPath = args.imagemagick_path + "/convert.exe"
    ku.compositePath = args.imagemagick_path + "/composite.exe"

ku.printHeader()
ku.printOutline()
ku.printHoles(False)
ku.printModules()
ku.generateImages()
